(*pp camlp4o pa_macro.cmo `cat CONFIG` *)

(*

  CamlTemplate: A template processor for Objective Caml programs.
  Copyright  2003, 2004 Benjamin Geer
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
  
  In addition, as a special exception, Benjamin Geer gives permission
  to link the code of this program with the Apache HTTP Server (or
  with modified versions of Apache that use the same license as
  Apache), and distribute linked combinations including the two. You
  must obey the GNU General Public License in all respects for all of
  the code used other than Apache. If you modify this file, you may
  extend this exception to your version of the file, but you are not
  obligated to do so. If you do not wish to do so, delete this
  exception statement from your version.

*)

(* $Id: ctUtil.ml,v 1.14 2004/07/15 19:41:01 ben Exp $ *)

(* Miscellaneous utility functions. *)


(* Returns the contents of a file as a string.  (Thanks to Markus
   Mottl.) *)
let read_file_as_string filename =
  let file = open_in filename in
  let size = in_channel_length file in
    try
      let buf = String.create size in
        really_input file buf 0 size;
        close_in file;
        buf
    with e ->
      (try close_in file with _ -> ());
      raise e

(* Implements try-with-finally. *)
let try_with_finally ~(f : unit -> 'a) ~(finally: unit -> unit) =
  let res =
    try
      f ()
    with e ->
      finally ();
      raise e
  in
    finally ();
    res ;;

(* If threads are enabled, creates a mutex, otherwise returns unit. *)
let create_mutex () =
  IFDEF THREADS THEN
    Mutex.create ()
  ELSE
    ()
  END ;;

(* Locks a mutex, calls a function, then unlocks the mutex.  The mutex
   is also unlocked if the function throws an exception. *)
IFDEF THREADS THEN
  let call_in_real_mutex ~(f : unit -> 'a) ~(mutex : Mutex.t) =
    Mutex.lock mutex;
    try_with_finally ~f ~finally:(function () -> Mutex.unlock mutex)
END ;;

(* This function's arguments are a function to be called, and a value
   returned by create_mutex.  If threads are enabled, it
   calls the function in a mutex.  If threads are disabled, it just
   calls the function. *)
let call_in_mutex ~(f : unit -> 'a) ~mutex =
  IFDEF THREADS THEN
    call_in_real_mutex ~f ~mutex
  ELSE
    f ()
  END ;;
